@@ -18,14 +18,12 @@
     reasons why the executable file might be covered by the GNU General Public License.
 */
 
+#include "../../common/utility/utility.h"
+
 #if __TBB_MIC_OFFLOAD
 #pragma offload_attribute (push,target(mic))
 #endif // __TBB_MIC_OFFLOAD
 
-// This header should come before any other one.
-// For details, see Known Issues in the Release Notes.
-#include "tbb/tbb_stddef.h"
-
 #include <cstdio>
 #include <cstdlib>
 #include <string>
@@ -35,8 +33,6 @@
 #include "tbb/task_scheduler_init.h"
 #include "tbb/task_group.h"
 
-#include "../../common/utility/utility.h"
-
 #pragma warning(disable: 4996)
 
 #if __INTEL_COMPILER
