@@ -1,38 +1,53 @@
 /*
-    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
+#include "harness_defs.h"
+
+#define HARNESS_DEFAULT_MIN_THREADS 2
+#define HARNESS_DEFAULT_MAX_THREADS 4
+
+#if _MSC_VER
+    #pragma warning (disable: 4503) // Suppress "decorated name length exceeded, name was truncated" warning
+    #if !TBB_USE_EXCEPTIONS
+        // Suppress "C++ exception handler used, but unwind semantics are not enabled" warning in STL headers
+        #pragma warning (disable: 4530)
+    #endif
+#endif
+
+#if __TBB_MSVC_UNREACHABLE_CODE_IGNORED
+    // Suppress "unreachable code" warning by VC++ 17.0-18.0 (VS 2012 or newer)
+    #pragma warning (disable: 4702)
+#endif
+
 #include "harness.h"
 
 // global task_scheduler_observer is an imperfect tool to find how many threads are really
 // participating.  That was the hope, but it counts the entries into the marketplace,
 // not the arena.
 // #define USE_TASK_SCHEDULER_OBSERVER 1
-#define TBB_PREVIEW_GRAPH_NODES 1
+
+#if _MSC_VER && defined(__INTEL_COMPILER) && !TBB_USE_DEBUG
+    #define TBB_RUN_BUFFERING_TEST __INTEL_COMPILER > 1210
+#else
+    #define TBB_RUN_BUFFERING_TEST 1
+#endif
 
 #if TBB_USE_EXCEPTIONS
 #if USE_TASK_SCHEDULER_OBSERVER
@@ -48,6 +63,7 @@
 inline intptr_t Existed() { return INT_MAX; }  // resolve Existed in harness_eh.h
 
 #include "harness_eh.h"
+#include <stdexcept>
 
 #define NUM_ITEMS 15
 int g_NumItems;
@@ -82,7 +98,11 @@ class WaitThrow;
 template<>
 class WaitThrow<serial_type,nonThrowing> {
 protected:
-    void WaitAndThrow(int /*cnt*/, const char * /*name*/) {
+    void WaitAndThrow(int cnt, const char * /*name*/) {
+        if(cnt > g_NumThreads + 10) {
+            Harness::ConcurrencyTracker ct;
+            WaitUntilConcurrencyPeaks();
+        }
     }
 };
 
@@ -285,10 +305,10 @@ struct less_body : public std::binary_function<ItemType,ItemType,bool> {
 // --------- tag methods for tag_matching join_node
 template<typename TT>
 class tag_func {
-    const TT my_mult;
-    tag_func& operator=( const tag_func& other);
+    TT my_mult;
 public:
     tag_func(TT multiplier) : my_mult(multiplier) { }
+    void operator=( const tag_func& other){my_mult = other.my_mult;}
     // operator() will return [0 .. Count) 
     tbb::flow::tag_value operator()( TT v) {
         tbb::flow::tag_value t = tbb::flow::tag_value(v / my_mult);
@@ -409,14 +429,20 @@ void run_source_node_test() {
 
 void test_source_node() {
     REMARK("Testing source_node\n");
-    check_type_counter = 0;
+    check_type<int>::check_type_counter = 0;
+    g_Wakeup_Msg = "source_node(1): Missed wakeup or machine is overloaded?";
     run_source_node_test<check_type<int>, isThrowing, nonThrowing>();
-    ASSERT(!check_type_counter, "Some items leaked in test");
+    ASSERT(!check_type<int>::check_type_counter, "Some items leaked in test");
+    g_Wakeup_Msg = "source_node(2): Missed wakeup or machine is overloaded?";
     run_source_node_test<int, isThrowing, nonThrowing>();
+    g_Wakeup_Msg = "source_node(3): Missed wakeup or machine is overloaded?";
     run_source_node_test<int, nonThrowing, isThrowing>();
+    g_Wakeup_Msg = "source_node(4): Missed wakeup or machine is overloaded?";
     run_source_node_test<int, isThrowing, isThrowing>();
+    g_Wakeup_Msg = "source_node(5): Missed wakeup or machine is overloaded?";
     run_source_node_test<check_type<int>, isThrowing, isThrowing>();
-    ASSERT(!check_type_counter, "Some items leaked in test");
+    g_Wakeup_Msg = g_Orig_Wakeup_Msg;
+    ASSERT(!check_type<int>::check_type_counter, "Some items leaked in test");
 }
 
 // -------- utilities & types to test function_node and multifunction_node.
@@ -441,6 +467,7 @@ struct AttachPoint<NodeType,ItemType,indx,func_node_type> {
     }
 };
 
+
 // common template for running function_node, multifunction_node.  continue_node
 // has different firing requirements, so it needs a different graph topology.
 template<
@@ -461,6 +488,8 @@ template<
 void
 run_one_functype_node_test(bool throwException, bool flog, const char * /*name*/) {
 
+    char mymsg[132];
+    char *saved_msg = const_cast<char *>(g_Wakeup_Msg);
     tbb::flow::graph g;
 
     tbb::atomic<int> source0_count;
@@ -487,6 +516,9 @@ run_one_functype_node_test(bool throwException, bool flog, const char * /*name*/
     make_edge(AttachPoint<TestNodeType, TypeToSink1, 1, NFT>::GetSender(node_to_test), sink1);
 
     for(int iter = 0; iter < 2; ++iter) {  // run, reset, run again
+        sprintf(mymsg, "%s iter=%d, threads=%d, throw=%s, flog=%s", saved_msg, iter, g_NumThreads,
+               throwException?"T":"F", flog?"T":"F");
+        g_Wakeup_Msg = mymsg;
         ResetGlobals(throwException,flog);
         if(throwException) {
             TRY();
@@ -522,15 +554,15 @@ run_one_functype_node_test(bool throwException, bool flog, const char * /*name*/
             ASSERT(t_cnt == 2*g_NumItems, "Not all items reached test node");
             ASSERT(nb0_cnt == 2*g_NumItems && nb1_cnt == 2*g_NumItems, "Missing items in absorbers");
         }
-    g.reset();  // resets the body of the source_nodes, test_node and the absorb_nodes. 
-    source0_count = source1_count = sink0_count = sink1_count = test_count = 0;
-    ASSERT(0 == tbb::flow::copy_body<SourceNodeBodyType0>(source0).count_value(),"Reset source 0 failed");
-    ASSERT(0 == tbb::flow::copy_body<SourceNodeBodyType1>(source1).count_value(),"Reset source 1 failed");
-    ASSERT(0 == tbb::flow::copy_body<TestNodeBodyType>(node_to_test).count_value(),"Reset test_node failed");
-    ASSERT(0 == tbb::flow::copy_body<SinkNodeBodyType0>(sink0).count_value(),"Reset sink 0 failed");
-    ASSERT(0 == tbb::flow::copy_body<SinkNodeBodyType1>(sink1).count_value(),"Reset sink 1 failed");
-
+        g.reset();  // resets the body of the source_nodes, test_node and the absorb_nodes. 
+        source0_count = source1_count = sink0_count = sink1_count = test_count = 0;
+        ASSERT(0 == tbb::flow::copy_body<SourceNodeBodyType0>(source0).count_value(),"Reset source 0 failed");
+        ASSERT(0 == tbb::flow::copy_body<SourceNodeBodyType1>(source1).count_value(),"Reset source 1 failed");
+        ASSERT(0 == tbb::flow::copy_body<TestNodeBodyType>(node_to_test).count_value(),"Reset test_node failed");
+        ASSERT(0 == tbb::flow::copy_body<SinkNodeBodyType0>(sink0).count_value(),"Reset sink 0 failed");
+        ASSERT(0 == tbb::flow::copy_body<SinkNodeBodyType1>(sink1).count_value(),"Reset sink 1 failed");
         
+        g_Wakeup_Msg = saved_msg;
     }
 #if USE_TASK_SCHEDULER_OBSERVER
     o.observe(false);
@@ -598,35 +630,44 @@ void run_function_node_test() {
 void test_function_node() {
     REMARK("Testing function_node\n");
     // serial rejecting
+    g_Wakeup_Msg = "function_node(1a): Missed wakeup or machine is overloaded?";
     run_function_node_test<isThrowing, nonThrowing, int, nonThrowing, int, nonThrowing, nonThrowing, tbb::flow::rejecting, serial_type>();
+    g_Wakeup_Msg = "function_node(1b): Missed wakeup or machine is overloaded?";
     run_function_node_test<nonThrowing, nonThrowing, int, isThrowing, int, nonThrowing, nonThrowing, tbb::flow::rejecting, serial_type>();
+    g_Wakeup_Msg = "function_node(1c): Missed wakeup or machine is overloaded?";
     run_function_node_test<nonThrowing, nonThrowing, int, nonThrowing, int, isThrowing, nonThrowing, tbb::flow::rejecting, serial_type>();
 
     // serial queueing
+    g_Wakeup_Msg = "function_node(2): Missed wakeup or machine is overloaded?";
     run_function_node_test<isThrowing, nonThrowing, int, nonThrowing, int, nonThrowing, nonThrowing, tbb::flow::queueing, serial_type>();
     run_function_node_test<nonThrowing, nonThrowing, int, isThrowing, int, nonThrowing, nonThrowing, tbb::flow::queueing, serial_type>();
     run_function_node_test<nonThrowing, nonThrowing, int, nonThrowing, int, isThrowing, nonThrowing, tbb::flow::queueing, serial_type>();
-    check_type_counter = 0;
+    check_type<int>::check_type_counter = 0;
     run_function_node_test<nonThrowing, nonThrowing, check_type<int>, nonThrowing, check_type<int>, isThrowing, nonThrowing, tbb::flow::queueing, serial_type>();
-    ASSERT(!check_type_counter, "Some items leaked in test");
+    ASSERT(!check_type<int>::check_type_counter, "Some items leaked in test");
 
     // unlimited parallel rejecting
+    g_Wakeup_Msg = "function_node(3): Missed wakeup or machine is overloaded?";
     run_function_node_test<isThrowing, nonThrowing, int, nonThrowing, int, nonThrowing, nonThrowing, tbb::flow::rejecting, unlimited_type>();
     run_function_node_test<nonThrowing, nonThrowing, int, isThrowing, int, nonThrowing, nonThrowing, tbb::flow::rejecting, unlimited_type>();
     run_function_node_test<nonThrowing, nonThrowing, int, nonThrowing, int, nonThrowing, isThrowing, tbb::flow::rejecting, unlimited_type>();
 
     // limited parallel rejecting
+    g_Wakeup_Msg = "function_node(4): Missed wakeup or machine is overloaded?";
     run_function_node_test<isThrowing, nonThrowing, int, nonThrowing, int, nonThrowing, nonThrowing, tbb::flow::rejecting, limited_type>();
     run_function_node_test<nonThrowing, nonThrowing, int, isThrowing, int, nonThrowing, nonThrowing, tbb::flow::rejecting, (size_t)limited_type>();
     run_function_node_test<nonThrowing, nonThrowing, int, nonThrowing, int, nonThrowing, isThrowing, tbb::flow::rejecting, (size_t)limited_type>();
 
     // limited parallel queueing
+    g_Wakeup_Msg = "function_node(5): Missed wakeup or machine is overloaded?";
     run_function_node_test<isThrowing, nonThrowing, int, nonThrowing, int, nonThrowing, nonThrowing, tbb::flow::queueing, (size_t)limited_type>();
     run_function_node_test<nonThrowing, nonThrowing, int, isThrowing, int, nonThrowing, nonThrowing, tbb::flow::queueing, (size_t)limited_type>();
     run_function_node_test<nonThrowing, nonThrowing, int, nonThrowing, int, nonThrowing, isThrowing, tbb::flow::queueing, (size_t)limited_type>();
 
     // everyone throwing
+    g_Wakeup_Msg = "function_node(6): Missed wakeup or machine is overloaded?";
     run_function_node_test<isThrowing, isThrowing, int, isThrowing, int, isThrowing, isThrowing, tbb::flow::rejecting, unlimited_type>();
+    g_Wakeup_Msg = g_Orig_Wakeup_Msg;
 }
 
 // ----------------------------------- multifunction_node ----------------------------------
@@ -695,36 +736,45 @@ void run_multifunction_node_test() {
 
 void test_multifunction_node() {
     REMARK("Testing multifunction_node\n");
+    g_Wakeup_Msg = "multifunction_node(source throws,rejecting,serial): Missed wakeup or machine is overloaded?";
     // serial rejecting
     run_multifunction_node_test<isThrowing, nonThrowing, int, nonThrowing, tbb::flow::tuple<int,float>, nonThrowing, nonThrowing, tbb::flow::rejecting, serial_type>();
+    g_Wakeup_Msg = "multifunction_node(test throws,rejecting,serial): Missed wakeup or machine is overloaded?";
     run_multifunction_node_test<nonThrowing, nonThrowing, int, isThrowing, tbb::flow::tuple<int,int>, nonThrowing, nonThrowing, tbb::flow::rejecting, serial_type>();
+    g_Wakeup_Msg = "multifunction_node(sink throws,rejecting,serial): Missed wakeup or machine is overloaded?";
     run_multifunction_node_test<nonThrowing, nonThrowing, int, nonThrowing, tbb::flow::tuple<int,int>, isThrowing, nonThrowing, tbb::flow::rejecting, serial_type>();
 
+    g_Wakeup_Msg = "multifunction_node(2): Missed wakeup or machine is overloaded?";
     // serial queueing
     run_multifunction_node_test<isThrowing, nonThrowing, int, nonThrowing, tbb::flow::tuple<int,int>, nonThrowing, nonThrowing, tbb::flow::queueing, serial_type>();
     run_multifunction_node_test<nonThrowing, nonThrowing, int, isThrowing, tbb::flow::tuple<int,int>, nonThrowing, nonThrowing, tbb::flow::queueing, serial_type>();
     run_multifunction_node_test<nonThrowing, nonThrowing, int, nonThrowing, tbb::flow::tuple<int,int>, isThrowing, nonThrowing, tbb::flow::queueing, serial_type>();
-    check_type_counter = 0;
+    check_type<int>::check_type_counter = 0;
     run_multifunction_node_test<nonThrowing, nonThrowing, check_type<int>, nonThrowing, tbb::flow::tuple<check_type<int>, check_type<int> >, isThrowing, nonThrowing, tbb::flow::queueing, serial_type>();
-    ASSERT(!check_type_counter, "Some items leaked in test");
+    ASSERT(!check_type<int>::check_type_counter, "Some items leaked in test");
 
+    g_Wakeup_Msg = "multifunction_node(3): Missed wakeup or machine is overloaded?";
     // unlimited parallel rejecting
     run_multifunction_node_test<isThrowing, nonThrowing, int, nonThrowing, tbb::flow::tuple<int,int>, nonThrowing, nonThrowing, tbb::flow::rejecting, unlimited_type>();
     run_multifunction_node_test<nonThrowing, nonThrowing, int, isThrowing, tbb::flow::tuple<int,int>, nonThrowing, nonThrowing, tbb::flow::rejecting, unlimited_type>();
     run_multifunction_node_test<nonThrowing, nonThrowing, int, nonThrowing, tbb::flow::tuple<int,int>, nonThrowing, isThrowing, tbb::flow::rejecting, unlimited_type>();
 
+    g_Wakeup_Msg = "multifunction_node(4): Missed wakeup or machine is overloaded?";
     // limited parallel rejecting
     run_multifunction_node_test<isThrowing, nonThrowing, int, nonThrowing, tbb::flow::tuple<int,int>, nonThrowing, nonThrowing, tbb::flow::rejecting, limited_type>();
     run_multifunction_node_test<nonThrowing, nonThrowing, int, isThrowing, tbb::flow::tuple<int,int>, nonThrowing, nonThrowing, tbb::flow::rejecting, (size_t)limited_type>();
     run_multifunction_node_test<nonThrowing, nonThrowing, int, nonThrowing, tbb::flow::tuple<int,int>, nonThrowing, isThrowing, tbb::flow::rejecting, (size_t)limited_type>();
 
+    g_Wakeup_Msg = "multifunction_node(5): Missed wakeup or machine is overloaded?";
     // limited parallel queueing
     run_multifunction_node_test<isThrowing, nonThrowing, int, nonThrowing, tbb::flow::tuple<int,int>, nonThrowing, nonThrowing, tbb::flow::queueing, (size_t)limited_type>();
     run_multifunction_node_test<nonThrowing, nonThrowing, int, isThrowing, tbb::flow::tuple<int,int>, nonThrowing, nonThrowing, tbb::flow::queueing, (size_t)limited_type>();
     run_multifunction_node_test<nonThrowing, nonThrowing, int, nonThrowing, tbb::flow::tuple<int,int>, nonThrowing, isThrowing, tbb::flow::queueing, (size_t)limited_type>();
 
+    g_Wakeup_Msg = "multifunction_node(6): Missed wakeup or machine is overloaded?";
     // everyone throwing
     run_multifunction_node_test<isThrowing, isThrowing, int, isThrowing, tbb::flow::tuple<int,int>, isThrowing, isThrowing, tbb::flow::rejecting, unlimited_type>();
+    g_Wakeup_Msg = g_Orig_Wakeup_Msg;
 }
 
 //
@@ -846,13 +896,18 @@ void run_continue_node_test() {
 // 
 void test_continue_node() {
     REMARK("Testing continue_node\n");
+    g_Wakeup_Msg = "buffer_node(non,is,non): Missed wakeup or machine is overloaded?";
     run_continue_node_test<int,nonThrowing,isThrowing,nonThrowing>();
+    g_Wakeup_Msg = "buffer_node(non,non,is): Missed wakeup or machine is overloaded?";
     run_continue_node_test<int,nonThrowing,nonThrowing,isThrowing>();
+    g_Wakeup_Msg = "buffer_node(is,non,non): Missed wakeup or machine is overloaded?";
     run_continue_node_test<int,isThrowing,nonThrowing,nonThrowing>();
+    g_Wakeup_Msg = "buffer_node(is,is,is): Missed wakeup or machine is overloaded?";
     run_continue_node_test<int,isThrowing,isThrowing,isThrowing>();
-    check_type_counter = 0;
+    check_type<double>::check_type_counter = 0;
     run_continue_node_test<check_type<double>,isThrowing,isThrowing,isThrowing>();
-    ASSERT(!check_type_counter, "Dropped objects in continue_node test");
+    ASSERT(!check_type<double>::check_type_counter, "Dropped objects in continue_node test");
+    g_Wakeup_Msg = g_Orig_Wakeup_Msg;
 }
 
 // ---------- buffer_node queue_node overwrite_node --------------
@@ -932,7 +987,6 @@ void run_one_buffer_node_test(bool throwException,bool flog) {
     o.observe(false);
 #endif
 }
-
 template<class BufferItemType,
          TestNodeTypeEnum SourceThrowType,
          TestNodeTypeEnum SinkThrowType>
@@ -950,6 +1004,7 @@ void run_buffer_queue_and_overwrite_node_test() {
         if(i == 2) continue;  // no need to test flog w/o throws
         bool throwException = (i & 0x1) != 0;
         bool doFlog = (i & 0x2) != 0;
+#if TBB_RUN_BUFFERING_TEST
         run_one_buffer_node_test<
             /* class BufferItemType*/     BufferItemType,
             /*class SourceNodeType*/      SrcType,
@@ -966,6 +1021,7 @@ void run_buffer_queue_and_overwrite_node_test() {
             /*class SinkNodeType*/        SnkType,
             /*class SinkNodeBodyType*/    SinkBodyType
             >(throwException, doFlog);
+#endif
         run_one_buffer_node_test<
             /* class BufferItemType*/     BufferItemType,
             /*class SourceNodeType*/      SrcType,
@@ -979,9 +1035,17 @@ void run_buffer_queue_and_overwrite_node_test() {
 
 void test_buffer_queue_and_overwrite_node() {
     REMARK("Testing buffer_node, queue_node and overwrite_node\n");
+#if TBB_RUN_BUFFERING_TEST
+#else
+    REMARK("skip buffer and queue test (known issue)\n");
+#endif
+    g_Wakeup_Msg = "buffer, queue, overwrite(is,non): Missed wakeup or machine is overloaded?";
     run_buffer_queue_and_overwrite_node_test<int,isThrowing,nonThrowing>();
+    g_Wakeup_Msg = "buffer, queue, overwrite(non,is): Missed wakeup or machine is overloaded?";
     run_buffer_queue_and_overwrite_node_test<int,nonThrowing,isThrowing>();
+    g_Wakeup_Msg = "buffer, queue, overwrite(is,is): Missed wakeup or machine is overloaded?";
     run_buffer_queue_and_overwrite_node_test<int,isThrowing,isThrowing>();
+    g_Wakeup_Msg = g_Orig_Wakeup_Msg;
 }
 
 // ---------- sequencer_node -------------------------
@@ -1095,11 +1159,15 @@ void run_sequencer_node_test() {
 
 void test_sequencer_node() {
     REMARK("Testing sequencer_node\n");
+    g_Wakeup_Msg = "sequencer_node(is,non): Missed wakeup or machine is overloaded?";
     run_sequencer_node_test<int, isThrowing,nonThrowing>();
-    check_type_counter = 0;
+    check_type<int>::check_type_counter = 0;
+    g_Wakeup_Msg = "sequencer_node(non,is): Missed wakeup or machine is overloaded?";
     run_sequencer_node_test<check_type<int>, nonThrowing,isThrowing>();
-    ASSERT(!check_type_counter, "Dropped objects in sequencer_node test");
+    ASSERT(!check_type<int>::check_type_counter, "Dropped objects in sequencer_node test");
+    g_Wakeup_Msg = "sequencer_node(is,is): Missed wakeup or machine is overloaded?";
     run_sequencer_node_test<int, isThrowing,isThrowing>();
+    g_Wakeup_Msg = g_Orig_Wakeup_Msg;
 }
 
 // ------------ priority_queue_node ------------------
@@ -1212,11 +1280,15 @@ void run_priority_queue_node_test() {
 
 void test_priority_queue_node() {
     REMARK("Testing priority_queue_node\n");
+    g_Wakeup_Msg = "priority_queue_node(is,non): Missed wakeup or machine is overloaded?";
     run_priority_queue_node_test<int, isThrowing,nonThrowing>();
-    check_type_counter = 0;
+    check_type<int>::check_type_counter = 0;
+    g_Wakeup_Msg = "priority_queue_node(non,is): Missed wakeup or machine is overloaded?";
     run_priority_queue_node_test<check_type<int>, nonThrowing,isThrowing>();
-    ASSERT(!check_type_counter, "Dropped objects in priority_queue_node test");
+    ASSERT(!check_type<int>::check_type_counter, "Dropped objects in priority_queue_node test");
+    g_Wakeup_Msg = "priority_queue_node(is,is): Missed wakeup or machine is overloaded?";
     run_priority_queue_node_test<int, isThrowing,isThrowing>();
+    g_Wakeup_Msg = g_Orig_Wakeup_Msg;
 }
 
 // ------------------- join_node ----------------
@@ -1304,7 +1376,7 @@ struct run_one_join_node_test {
             if(iter == 0) {
                 remove_edge(node_to_test, sink);
                 tbb::flow::input_port<0>(node_to_test).try_put(ItemType0(g_NumItems + 1));
-                tbb::flow::input_port<0>(node_to_test).try_put(ItemType0(g_NumItems + 2));
+                tbb::flow::input_port<1>(node_to_test).try_put(ItemType1(g_NumItems + 2));
                 g.wait_for_all();
                 g.reset();
                 source0_count = source1_count = sink_count = 0;
@@ -1408,7 +1480,7 @@ struct run_one_join_node_test<
             if(iter == 0) {
                 remove_edge(node_to_test, sink);
                 tbb::flow::input_port<0>(node_to_test).try_put(ItemType0(g_NumItems + 4));
-                tbb::flow::input_port<0>(node_to_test).try_put(ItemType0(g_NumItems + 2));
+                tbb::flow::input_port<1>(node_to_test).try_put(ItemType1(g_NumItems + 2));
                 g.wait_for_all();   // have to wait for the graph to stop again....
                 g.reset();  // resets the body of the source_nodes, test_node and the absorb_nodes. 
                 source0_count = source1_count = sink_count = 0;
@@ -1468,11 +1540,15 @@ template<tbb::flow::graph_buffer_policy JP>
 void test_join_node() {
     REMARK("Testing join_node<%s>\n", graph_policy_name<JP>::name());
     // only doing two-input joins
+    g_Wakeup_Msg = "join(is,non): Missed wakeup or machine is overloaded?";
     run_join_node_test<JP, tbb::flow::tuple<int,int>,  isThrowing, nonThrowing>();
-    check_type_counter = 0;
+    check_type<int>::check_type_counter = 0;
+    g_Wakeup_Msg = "join(non,is): Missed wakeup or machine is overloaded?";
     run_join_node_test<JP, tbb::flow::tuple<check_type<int>,int>, nonThrowing, isThrowing>();
-    ASSERT(!check_type_counter, "Dropped items in test");
+    ASSERT(!check_type<int>::check_type_counter, "Dropped items in test");
+    g_Wakeup_Msg = "join(is,is): Missed wakeup or machine is overloaded?";
     run_join_node_test<JP, tbb::flow::tuple<int,int>,  isThrowing, isThrowing>();
+    g_Wakeup_Msg = g_Orig_Wakeup_Msg;
 }
 
 // ------------------- limiter_node -------------
@@ -1584,9 +1660,13 @@ void run_limiter_node_test() {
 
 void test_limiter_node() {
     REMARK("Testing limiter_node\n");
+    g_Wakeup_Msg = "limiter_node(is,non): Missed wakeup or machine is overloaded?";
     run_limiter_node_test<int,isThrowing,nonThrowing>();
+    g_Wakeup_Msg = "limiter_node(non,is): Missed wakeup or machine is overloaded?";
     run_limiter_node_test<int,nonThrowing,isThrowing>();
+    g_Wakeup_Msg = "limiter_node(is,is): Missed wakeup or machine is overloaded?";
     run_limiter_node_test<int,isThrowing,isThrowing>();
+    g_Wakeup_Msg = g_Orig_Wakeup_Msg;
 }
 
 // -------- split_node --------------------
@@ -1697,13 +1777,16 @@ void run_split_node_test() {
 
 void test_split_node() {
     REMARK("Testing split_node\n");
+    g_Wakeup_Msg = "split_node(is,non): Missed wakeup or machine is overloaded?";
     run_split_node_test<tbb::flow::tuple<int,int>, isThrowing, nonThrowing>();
+    g_Wakeup_Msg = "split_node(non,is): Missed wakeup or machine is overloaded?";
     run_split_node_test<tbb::flow::tuple<int,int>, nonThrowing, isThrowing>();
+    g_Wakeup_Msg = "split_node(is,is): Missed wakeup or machine is overloaded?";
     run_split_node_test<tbb::flow::tuple<int,int>, isThrowing,  isThrowing>();
+    g_Wakeup_Msg = g_Orig_Wakeup_Msg;
 }
 
-#if TBB_PREVIEW_GRAPH_NODES
-// --------- or_node ----------------------
+// --------- indexer_node ----------------------
 
 template < class InputTuple,
     class SourceType0,
@@ -1713,7 +1796,7 @@ template < class InputTuple,
     class TestNodeType,
     class SinkType,
     class SinkBodyType>
-void run_one_or_node_test(bool throwException,bool flog) {
+void run_one_indexer_node_test(bool throwException,bool flog) {
     typedef typename tbb::flow::tuple_element<0,InputTuple>::type ItemType0;
     typedef typename tbb::flow::tuple_element<1,InputTuple>::type ItemType1;
 
@@ -1771,7 +1854,7 @@ void run_one_or_node_test(bool throwException,bool flog) {
         if(iter == 0) {
             remove_edge(node_to_test, sink);
             tbb::flow::input_port<0>(node_to_test).try_put(ItemType0(g_NumItems + 4));
-            tbb::flow::input_port<0>(node_to_test).try_put(ItemType0(g_NumItems + 2));
+            tbb::flow::input_port<1>(node_to_test).try_put(ItemType1(g_NumItems + 2));
             g.wait_for_all();
             g.reset();
             source0_count = source1_count = sink_count = 0;
@@ -1797,12 +1880,12 @@ void run_one_or_node_test(bool throwException,bool flog) {
 template<class InputTuple,
     TestNodeTypeEnum SourceThrowType,
     TestNodeTypeEnum SinkThrowType>
-void run_or_node_test() {
+void run_indexer_node_test() {
     typedef typename tbb::flow::tuple_element<0,InputTuple>::type ItemType0;
     typedef typename tbb::flow::tuple_element<1,InputTuple>::type ItemType1;
     typedef test_source_body<ItemType0,SourceThrowType> SourceBodyType0;
     typedef test_source_body<ItemType1,SourceThrowType> SourceBodyType1;
-    typedef typename tbb::flow::or_node<InputTuple> TestNodeType;
+    typedef typename tbb::flow::indexer_node<ItemType0, ItemType1> TestNodeType;
     typedef absorber_body<typename TestNodeType::output_type,tbb::flow::continue_msg,SinkThrowType,unlimited_type> SinkBodyType;
 
     typedef typename tbb::flow::source_node<ItemType0> SourceType0;
@@ -1813,7 +1896,7 @@ void run_or_node_test() {
         if(2 == i) continue;
         bool throwException = (i & 0x1) != 0;
         bool doFlog = (i & 0x2) != 0;
-        run_one_or_node_test<
+        run_one_indexer_node_test<
              InputTuple,
              SourceType0,
              SourceBodyType0,
@@ -1825,13 +1908,16 @@ void run_or_node_test() {
     }
 }
 
-void test_or_node() {
-    REMARK("Testing or_node\n");
-    run_or_node_test<tbb::flow::tuple<int,int>, isThrowing, nonThrowing>();
-    run_or_node_test<tbb::flow::tuple<int,int>, nonThrowing, isThrowing>();
-    run_or_node_test<tbb::flow::tuple<int,int>, isThrowing,  isThrowing>();
+void test_indexer_node() {
+    REMARK("Testing indexer_node\n");
+    g_Wakeup_Msg = "indexer_node(is,non): Missed wakeup or machine is overloaded?";
+    run_indexer_node_test<tbb::flow::tuple<int,int>, isThrowing, nonThrowing>();
+    g_Wakeup_Msg = "indexer_node(non,is): Missed wakeup or machine is overloaded?";
+    run_indexer_node_test<tbb::flow::tuple<int,int>, nonThrowing, isThrowing>();
+    g_Wakeup_Msg = "indexer_node(is,is): Missed wakeup or machine is overloaded?";
+    run_indexer_node_test<tbb::flow::tuple<int,int>, isThrowing,  isThrowing>();
+    g_Wakeup_Msg = g_Orig_Wakeup_Msg;;
 }
-#endif
 
 ///////////////////////////////////////////////
 // whole-graph exception test
@@ -1915,7 +2001,6 @@ void TestOneThreadNum(int nThread) {
         test_function_node();
         test_continue_node();  // also test broadcast_node
         test_multifunction_node();
-
         // single- and multi-item buffering nodes
         test_buffer_queue_and_overwrite_node();
         test_sequencer_node();
@@ -1932,11 +2017,7 @@ void TestOneThreadNum(int nThread) {
         // not do try_puts after it has been set.  To get parallelism of N we have
         // to attach N successor nodes to the write_once (or play some similar game).
         // test_write_once_node();
-#if TBB_PREVIEW_GRAPH_NODES
-        test_or_node();
-#else
-        REMARK("or_node test skipped\n");
-#endif
+        test_indexer_node();
     }
 }
 #endif // TBB_USE_EXCEPTIONS
