@@ -1,31 +1,26 @@
 /*
-    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
+#define HARNESS_DEFAULT_MIN_THREADS 0
+#define HARNESS_DEFAULT_MAX_THREADS 4
+
 #include "tbb/enumerable_thread_specific.h"
 #include "tbb/task_scheduler_init.h"
 #include "tbb/parallel_for.h"
@@ -63,8 +58,15 @@
 static tbb::atomic<int> construction_counter;
 static tbb::atomic<int> destruction_counter;
 
+#if TBB_USE_DEBUG
+const int REPETITIONS = 4;
+const int N = 10000;
+const int RANGE_MIN=1000;
+#else
 const int REPETITIONS = 10;
 const int N = 100000;
+const int RANGE_MIN=10000;
+#endif
 const int VALID_NUMBER_OF_KEYS = 100;
 const double EXPECTED_SUM = (REPETITIONS + 1) * N;
 
@@ -278,15 +280,15 @@ void run_parallel_scalar_tests_nocombine(const char *test_name) {
             ets_type finit_ets(my_finit);
 
             ASSERT( sums.empty(), NULL);
-            tbb::parallel_for( tbb::blocked_range<int>( 0, N, 10000 ), parallel_scalar_body<T>( sums ) );
+            tbb::parallel_for( tbb::blocked_range<int>( 0, N, RANGE_MIN ), parallel_scalar_body<T>( sums ) );
             ASSERT( !sums.empty(), NULL);
 
             ASSERT( finit_ets.empty(), NULL);
-            tbb::parallel_for( tbb::blocked_range<int>( 0, N, 10000 ), parallel_scalar_body<T>( finit_ets ) );
+            tbb::parallel_for( tbb::blocked_range<int>( 0, N, RANGE_MIN ), parallel_scalar_body<T>( finit_ets ) );
             ASSERT( !finit_ets.empty(), NULL);
 
             ASSERT(static_sums.empty(), NULL);
-            tbb::parallel_for( tbb::blocked_range<int>( 0, N, 10000 ), parallel_scalar_body<T>( static_sums ) );
+            tbb::parallel_for( tbb::blocked_range<int>( 0, N, RANGE_MIN ), parallel_scalar_body<T>( static_sums ) );
             ASSERT( !static_sums.empty(), NULL);
 
             // use iterator
@@ -411,11 +413,11 @@ void run_parallel_scalar_tests(const char *test_name) {
             ets_type sums(exemplar);
 
             ASSERT( sums.empty(), NULL);
-            tbb::parallel_for( tbb::blocked_range<int>( 0, N, 10000 ), parallel_scalar_body<T>( sums ) );
+            tbb::parallel_for( tbb::blocked_range<int>( 0, N, RANGE_MIN ), parallel_scalar_body<T>( sums ) );
             ASSERT( !sums.empty(), NULL);
 
             ASSERT(static_sums.empty(), NULL);
-            tbb::parallel_for( tbb::blocked_range<int>( 0, N, 10000 ), parallel_scalar_body<T>( static_sums ) );
+            tbb::parallel_for( tbb::blocked_range<int>( 0, N, RANGE_MIN ), parallel_scalar_body<T>( static_sums ) );
             ASSERT( !static_sums.empty(), NULL);
 
 
@@ -502,7 +504,7 @@ void run_parallel_vector_tests(const char *test_name) {
             ets_type vs;
 
             ASSERT( vs.empty(), NULL);
-            tbb::parallel_for ( tbb::blocked_range<int> (0, N, 10000), parallel_vector_for_body<T>( vs ) );
+            tbb::parallel_for ( tbb::blocked_range<int> (0, N, RANGE_MIN), parallel_vector_for_body<T>( vs ) );
             ASSERT( !vs.empty(), NULL);
 
             // copy construct
@@ -559,7 +561,7 @@ void run_cross_type_vector_tests(const char *test_name) {
             ets_nokey_type vs;
 
             ASSERT( vs.empty(), NULL);
-            tbb::parallel_for ( tbb::blocked_range<int> (0, N, 10000), parallel_vector_for_body<T>( vs ) );
+            tbb::parallel_for ( tbb::blocked_range<int> (0, N, RANGE_MIN), parallel_vector_for_body<T>( vs ) );
             ASSERT( !vs.empty(), NULL);
 
             // copy construct
@@ -1002,6 +1004,23 @@ void TestInstantiation() {
     ets3.combine(HasNoDefaultConstructorCombine());
 }
 
+class BigType {
+public:
+    BigType() { /* avoid cl warning C4345 about default initialization of POD types */ }
+    char my_data[12 * 1024 * 1024];
+};
+
+void TestConstructor() {
+    typedef tbb::enumerable_thread_specific<BigType> CounterBigType;
+    // Test default constructor
+    CounterBigType MyCounters; 
+    // Create a local instance.
+    CounterBigType::reference my_local = MyCounters.local();
+    my_local.my_data[0] = 'a';
+    // Test copy constructor
+    CounterBigType MyCounters2(MyCounters);
+}
+
 int TestMain () {
     TestInstantiation();
     run_segmented_iterator_tests();
@@ -1018,5 +1037,7 @@ int TestMain () {
 
     run_assignment_and_copy_constructor_tests();
 
+    TestConstructor();
+
     return Harness::Done;
 }
